/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: WSNode.java,v 1.17 2007/05/19 02:45:24 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.ws;

import javax.xml.namespace.QName;

import org.xmlpull.infoset.XmlElement;

import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.ws.WSComponent;
import edu.indiana.extreme.xbaya.component.ws.WSComponentFactory;
import edu.indiana.extreme.xbaya.graph.Edge;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.GraphSchema;
import edu.indiana.extreme.xbaya.graph.gui.NodeGUI;
import edu.indiana.extreme.xbaya.graph.impl.NodeImpl;
import edu.indiana.extreme.xbaya.graph.util.GraphUtil;
import edu.indiana.extreme.xbaya.graph.ws.gui.WSNodeGUI;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;

/**
 * @author Satoshi Shirasuna
 */
public class WSNode extends NodeImpl {

    // private static final MLogger logger = MLogger.getLogger();

    // @Deprecated
    // private QName wsdlQName;

    private String wsdlID;

    private QName portTypeQName;

    private String operationName;

    private WSNodeGUI gui;

    /**
     * Constructs a WsdlNode.
     * 
     * @param nodeElement
     * @throws GraphException
     */
    public WSNode(XmlElement nodeElement) throws GraphException {
        super(nodeElement);
    }

    /**
     * Constructs a WSNode.
     * 
     * @param graph
     */
    public WSNode(Graph graph) {
        super(graph);
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Node#getGUI()
     */
    public NodeGUI getGUI() {
        if (this.gui == null) {
            this.gui = new WSNodeGUI(this);
        }
        return this.gui;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.Node#getComponent()
     */
    @Override
    public WSComponent getComponent() {
        return (WSComponent) super.getComponent();
    }

    /**
     * Returns the WSDL ID.
     * 
     * @return the WSDL ID
     */
    public String getWSDLID() {
        return this.wsdlID;
    }

    /**
     * @param wsdlID
     */
    public void setWSDLID(String wsdlID) {
        this.wsdlID = wsdlID;
    }

    /**
     * @return The name of portType.
     */
    public QName getPortTypeQName() {
        if (this.portTypeQName == null) {
            if (getComponent() == null) {
                // XXX This happens while parsing xwf created by the version
                // 2.2.6_1 or below.
                return null;
            }
            this.portTypeQName = getComponent().getPortTypeQName();
        }
        return this.portTypeQName;
    }

    /**
     * @return The name of the operation.
     */
    public String getOperationName() {
        if (this.operationName == null) {
            if (getComponent() == null) {
                // XXX This happens while parsing xwf created by the version
                // 2.2.6_1 or below.
                return null;
            }
            this.operationName = getComponent().getOperationName();
        }
        return this.operationName;
    }

    /**
     * @throws GraphException
     * @see edu.indiana.extreme.xbaya.graph.impl.NodeImpl#edgeWasAdded(edu.indiana.extreme.xbaya.graph.Edge)
     */
    @Override
    protected void edgeWasAdded(Edge edge) throws GraphException {
        GraphUtil.validateConnection(edge);
    }

    /**
     * @return the node xml
     */
    @Override
    protected XmlElement toXML() {
        XmlElement nodeElement = super.toXML();
        nodeElement.setAttributeValue(GraphSchema.NS,
                GraphSchema.NODE_TYPE_ATTRIBUTE, GraphSchema.NODE_TYPE_WS);

        XmlElement wsdlElement = nodeElement.addElement(GraphSchema.NS,
                GraphSchema.NODE_WSDL_QNAME_TAG);
        // wsdlElement.setText(getWSDLQName().toString());
        wsdlElement.setText(getWSDLID());

        XmlElement portTypeElement = nodeElement.addElement(GraphSchema.NS,
                GraphSchema.NODE_WSDL_PORT_TYPE_TAG);
        portTypeElement.setText(getPortTypeQName().toString());

        XmlElement operationElement = nodeElement.addElement(GraphSchema.NS,
                GraphSchema.NODE_WSDL_OPERATION_TAG);
        operationElement.setText(getOperationName());

        return nodeElement;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.impl.NodeImpl#parse(org.xmlpull.infoset.XmlElement)
     */
    @Override
    protected void parse(XmlElement nodeElement) throws GraphException {
        super.parse(nodeElement);

        XmlElement wsdlElement = nodeElement.element(null,
                GraphSchema.NODE_WSDL_QNAME_TAG);
        if (wsdlElement != null) {
            this.wsdlID = wsdlElement.requiredText();
            // String wsdlQNameString = wsdlElement.requiredText();
            // this.wsdlQName = QName.valueOf(wsdlQNameString);
        }

        XmlElement portTypeElement = nodeElement.element(null,
                GraphSchema.NODE_WSDL_PORT_TYPE_TAG);
        if (portTypeElement != null) {
            String portTypeString = portTypeElement.requiredText();
            this.portTypeQName = QName.valueOf(portTypeString);
        }

        XmlElement operationElement = nodeElement.element(null,
                GraphSchema.NODE_WSDL_OPERATION_TAG);
        if (operationElement != null) {
            this.operationName = operationElement.requiredText();
        }
    }

    @Override
    @Deprecated
    protected void parseComponent(XmlElement componentElement)
            throws GraphException {
        try {
            String componentString = componentElement.requiredText();
            WSComponent wsdlComponent = WSComponentFactory
                    .createComponent(componentString);
            setComponent(wsdlComponent);
        } catch (ComponentException e) {
            throw new GraphException(ErrorMessages.COMPONENT_FORMAT_ERROR, e);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
